@@ -106,6 +106,11 @@ eog_image_dispose (GObject *object)
 		g_free (priv->error_message);
 		priv->error_message = NULL;
 	}
+
+	if (priv->mutex) {
+		g_mutex_free (priv->mutex);
+		priv->mutex = NULL;
+	}
 	
 	if (priv->status_mutex) {
 		g_mutex_free (priv->status_mutex);
@@ -257,6 +262,7 @@ eog_image_init (EogImage *img)
 	priv->thumbnail = NULL;
 	priv->width = priv->height = -1;
 	priv->modified = FALSE;
+	priv->mutex = g_mutex_new ();
 	priv->status_mutex = g_mutex_new ();
 	priv->load_finished = NULL;
 	priv->error_message = NULL;
@@ -271,6 +277,24 @@ eog_image_init (EogImage *img)
 	img->priv = priv;
 }
 
+void eog_image_lock (EogImage *image)
+{
+        g_return_if_fail (EOG_IS_IMAGE (image));
+	g_return_if_fail (image->priv != NULL);
+	g_return_if_fail (image->priv->mutex != NULL);
+
+	g_mutex_lock (image->priv->mutex);
+}
+
+void eog_image_unlock (EogImage *image)
+{
+        g_return_if_fail (EOG_IS_IMAGE (image));
+	g_return_if_fail (image->priv != NULL);
+	g_return_if_fail (image->priv->mutex != NULL);
+
+	g_mutex_unlock (image->priv->mutex);
+}
+
 EogImage* 
 eog_image_new_uri (GnomeVFSURI *uri)
 {
@@ -1674,7 +1698,7 @@ eog_image_get_collate_key (EogImage *img)
 		char *caption;
 
 		caption = eog_image_get_caption (img);
-		priv->caption_key = g_utf8_collate_key (caption, -1);
+		priv->caption_key = g_utf8_collate_key_for_filename (caption, -1);
 	}
 
 	return priv->caption_key;
@@ -1692,6 +1716,7 @@ eog_image_cancel_load (EogImage *img)
 	g_mutex_lock (priv->status_mutex);
 	if (priv->status == EOG_IMAGE_STATUS_LOADING) {
 		priv->cancel_loading = TRUE;
+		priv->status = EOG_IMAGE_STATUS_UNKNOWN;
 	}
 	g_mutex_unlock (priv->status_mutex);
 }
@@ -1893,9 +1918,13 @@ eog_image_print (EogImage *img, GnomePrintContext *context, gdouble paper_width,
 	gnome_print_translate (context, (paper_width - width)/2.0, (paper_height - height)/2.0);
 	gnome_print_scale (context, width, height);
 	
-	gnome_print_rgbimage (context, gdk_pixbuf_get_pixels (printed_image), 
-		pix_width, pix_height, gdk_pixbuf_get_rowstride (printed_image));
-		
+	if (gdk_pixbuf_get_has_alpha (printed_image)) {
+		gnome_print_rgbaimage (context, gdk_pixbuf_get_pixels (printed_image), 
+			pix_width, pix_height, gdk_pixbuf_get_rowstride (printed_image));
+	} else {
+		gnome_print_rgbimage (context, gdk_pixbuf_get_pixels (printed_image), 
+			pix_width, pix_height, gdk_pixbuf_get_rowstride (printed_image));
+	}
   
 	g_object_unref (G_OBJECT (printed_image));	
 }
